#include <stdio.h>
#include <fcntl.h>
#include <sys/sblast.h>
#include "au_file.h"
#include "ulaw.h"

int
read_audio_header(int a, Audio_hdr *buf)
{
	read(a, buf, sizeof(Audio_hdr));

	buf->magic = ntohl(buf->magic);
	buf->hdr_size = ntohl(buf->hdr_size);
	buf->data_size = ntohl(buf->data_size);
	buf->data_encoding = ntohl(buf->data_encoding);
	buf->sample_rate = ntohl(buf->sample_rate);
	buf->channels = ntohl(buf->channels);

	printf("magic %x\nhdr_size %d\ndata size %d\n",
		buf->magic, buf->hdr_size, buf->data_size);
	printf("type %d (%s)\n",
		buf->data_encoding, (buf->data_encoding == AUDIO_FILE_ENCODING_MULAW_8)
		? "8-bit ISDN u-law" : "something");
	printf("sample_rate %d, channels %d\n\n", buf->sample_rate, buf->channels);

	lseek(a, buf->hdr_size, SEEK_SET);

	return buf->magic == AUDIO_FILE_MAGIC;
}

#define BSIZE (8 * 1024)

/*
 * Send an ioctl to the DSP with error checking.
 */
#define IOCTL(cmd, arg) \
  { \
    if (ioctl (dsp, DSP_IOCTL_##cmd, arg) == -1) \
        perror ("ioctl"), exit(4); \
  }

int
open_dsp(Audio_hdr *ahdr)
{
	int dsp;
	int zero = 0;
	int one = 1;
	
	dsp = open("/dev/sb_dsp", O_RDWR);
	if (dsp < 1) {
		perror("Can't open /dev/sb_dsp");
		exit(5);
	}
	IOCTL(STEREO, (ahdr->channels == 1) ? &zero : &one);
	IOCTL(SPEED, &ahdr->sample_rate);

	return dsp;
}

main(int ac, char **av)
{
	Audio_hdr ahdr;
	unsigned char buf[BSIZE];
	int audio;
	int len;
	int i, fd;

	if (ac < 2)
		fd = 0;
	else
		fd = open(av[1], O_RDONLY);
	if (read_audio_header(fd, &ahdr) == 0) {
		fprintf(stderr, "Bad file format\n");
		exit(1);
	}

	if (ahdr.data_encoding != AUDIO_FILE_ENCODING_MULAW_8 
	  && ahdr.data_encoding != AUDIO_FILE_ENCODING_LINEAR_8) {
		fprintf(stderr, "Can only handle 8bit MULAW or LINEAR encoding\n");
		exit(8);
	}
	audio = open_dsp(&ahdr);

	while((len = read(fd, buf, BSIZE)) > 0) {
		if (ahdr.data_encoding == AUDIO_FILE_ENCODING_MULAW_8) {
			for(i = 0; i < len; i++) {
				buf[i] = (char) 128 + (ulaw_to_linear(buf[i]) / 256);
			}
		}

		if (len != write(audio, buf, len)) {
			perror("Write to dsp failure");
			exit(2);
		}
		printf("... %d\n", len);
	}
	close(audio);
	exit(0);
}
