/******************************************************************************
   			        Voxclock - NetBSD Speaking Clock

    	    Copyright 1993 Martin H. Smith All Rights Reserved.
*******************************************************************************

Project		: VOXCLOCK
Filename	: clock.c
Author		: M.H. Smith (msmith@lssec.bt.co.uk)
Purpose		: Tells the time believe it or not
Created		: 19th September 1993
RCSid		: $Id: clock.c,v 1.9 1993/09/22 20:34:20 msmith Exp $

Notes:

This program may be freely distributed and copied in source and binary form as
long as it is not sold and the header and copyright information is included in
all such copies.

Every care has been taken in the preparation of this software. It is used at
the recipients risk without charge and I can accept no responsibility for loss
or damage howsoever caused.

******************************************************************************/

#include <fcntl.h>
#include <time.h> 
#include <syslog.h>

/*
 * I hate having to hard code this but if I use an include dir in the makefile
 * it will pick up /sys/sys/time.h and things screw up.
 */

#include "/sys/sys/sblast.h"

#include "voxclock.h"

extern boolean want_ampm;						  /* all defined in main.c */
extern boolean half_hour;
extern boolean chime_all;
extern boolean speaking;
extern boolean test_mode;

extern int chime_play_rate;
extern int chime_data_len;
extern unsigned char *chime_data_ptr;

extern int voice_play_rate;


/*
 * Work out how many seconds until we need to wake up and think about doing
 * something
 */

static int time_to_event()

{
	time_t timenow = time(NULL);				  /* what time is it? */
	struct tm * tp = localtime(&timenow);		  /* what time is it really? */
	int target_h = tp -> tm_hour;
	int target_m;
	int res = 0;
	
	if (tp -> tm_min < 30 && half_hour)
		target_m = 30;							  /* chime on half hour ? */
	else
	{
		if (tp -> tm_min)						  /* might be due now */
			++target_h;
		
		target_m = 0;							  /* must be on the hour */
	}

	res = (target_h * 3600 + target_m * 60) -
		(tp -> tm_hour * 3600 + tp -> tm_min * 60 + tp -> tm_sec);

	return res;									  /* no of seconds to go */
}


/*
 * Wait until we can open the soundblaster device and return the file
 * descriptor we got
 */

static int open_soundblaster(void)

{
	int fd = -1;
	int tries = 0;

	while (fd < 0 && tries++ < SB_OPEN_TRIES)
	{
		fd = open(SB_DSP_DEV, O_RDWR, 0);

		if (fd < 0)
			sleep(SB_OPEN_SLEEP);
	}

	if (fd < 0)
		syslog(LOG_ERR, "Failed to open soundblaster device");
	
	return fd;
}


/*
 * Initialise soundblaster
 */

static void init_soundblaster(int fd, int speed)

{
	int arg = 0;
	
	if (ioctl(fd, DSP_IOCTL_STEREO, &arg) < 0)
		syslog(LOG_ERR, "Failed to select mono with ioctl!\n");

	arg = speed;

	if (ioctl(fd, DSP_IOCTL_SPEED, &arg) < 0)
		syslog(LOG_ERR, "Failed to select speed with ioctl!\n");
}


/*
 * Play either one or more chimes on the soundblaster depending on the
 * selection
 */

static void big_ben(int fd)						  /* apologies for the name */

{
	time_t thetime = time(NULL);
	struct tm * tp = localtime(&thetime);
	int bong;

	int chimes = chime_all ? (tp -> tm_hour % 12) : 1;

	if (!chimes)
		chimes = 12;							  /* midnight */

	if (tp -> tm_min == 30)
		chimes = 1;								  /* once on the half hour */

	for (bong = 0; bong < chimes; ++bong)		  /* here is the news */
		write(fd, chime_data_ptr, chime_data_len);
}


/*
 * Speak one particular word, bung out the sample to the DSP device
 */

static void speak_word(int fd, word_enum what)

{
	int length;
	String ptr = get_sample_data(what, &length);

	write(fd, ptr, length);
}

	
/*
 * Say the time in a nice friendly sort of way!
 */

static void speak_the_time(int fd)

{
	time_t thetime = time(NULL);
	struct tm * tp = localtime(&thetime);
	int hour = tp -> tm_hour % 12;
	word_enum ampm_word = (tp -> tm_hour >= 12) ? W_PM : W_AM;
	
	if (!hour)
		hour = 12;								  /* midnight, not zero */

	speak_word(fd, W_ITS);						  /* Monty Pythons Flying
													 Circus? */
	speak_word(fd, W_1 + hour - 1);				  /* say the hour */

	if (want_ampm)								  /* say AM or PM */
	{
		if (tp -> tm_min == 30 && half_hour)	  /* half hour */
			speak_word(fd, W_30);
		
		speak_word(fd, ampm_word);				  /* say right word */
	}
	else
	{
		if (tp -> tm_min != 30)
			speak_word(fd, W_OCLOCK);			  /* don't say it half hour */
	}
}


/*
 * Announce the time, open the DSP device on the soundblaster and select the
 * correct speed. Then play the right number of chimes. Then select the VOX
 * speed and announce the time if necessary
 */

static void announce_time(void)

{
	int fd = open_soundblaster();

	if (fd > 0)
	{
		init_soundblaster(fd, chime_play_rate);	  /* ready to chime */
	
		big_ben(fd);							  /* do chimes */
		
		ioctl(fd, DSP_IOCTL_FLUSH, 0);
		close(fd);
	}
	
	if (speaking)								  /* want an announcement */
	{
		fd = open_soundblaster();

		if (fd > 0)
		{
			init_soundblaster(fd, voice_play_rate);	/* ready to speak */
			
			speak_the_time(fd);

			close(fd);
		}
	}
}


/*
 * Main clock entrypoint, an infinite loop. This is of course totally
 * deliberate and I never write them by accident. It was always the person on
 * holiday.
 */

void clock_go(void)

{
	while (1)									  /* forever is a long time */
	{
		int sleepfor = time_to_event();			  /* wait interval */

		if (test_mode)
			sleepfor = 5;						  /* impatience rules */

		sleep(sleepfor);						  /* wait for time */

		announce_time();						  /* make a song and dance */
	}
}

/* file identification */

static char RCSid[] = "$Id: clock.c,v 1.9 1993/09/22 20:34:20 msmith Exp $";
