/******************************************************************************
                    Voxclock - NetBSD Speaking Clock

    	    Copyright 1993 Martin H. Smith All Rights Reserved.
*******************************************************************************

Project		: VOXCLOCK
Filename	: VOXCLOCK.H
Author		: M.H. Smith (msmith@lssec.bt.co.uk)
Purpose		: Definitions for voxclock program
Created		: 10th June 1993
RCSid		: $Id: voxclock.h,v 1.4 1993/09/22 19:31:29 msmith Exp $

Notes:

You need to have a SoundBlaster installed and the device driver written by
Steve Haehnichen <shaehnic@ucsd.edu> built into the kernel to make this program
work.

This program may be freely distributed and copied in source and binary form as
long as it is not sold and the header and copyright information is included in
all such copies.

Every care has been taken in the preparation of this software. It is used at
the recipients risk without charge and I can accept no responsibility for loss
or damage howsoever caused.

******************************************************************************/

#ifndef VOXCLOCK_H_INC
#define VOXCLOCK_H_INC

#include <stdio.h>

#define VOX_VERSION "V1.01"

typedef char * String;
typedef FILE * Stream;
typedef enum { FALSE, TRUE } boolean;

typedef int int32;
typedef unsigned int Uint32;

#ifndef DEFAULT_CHIME
#define DEFAULT_CHIME "/usr/local/lib/voxclock/chime"
#endif

#ifndef DEFAULT_VOICE
#define DEFAULT_VOICE "/usr/local/lib/voxclock/voice"
#endif

#define VOX_FILE_ID "VOXB"						  /* file id string */

#define SB_DSP_DEV "/dev/sb_dsp"				  /* DSP device file */
#define SB_OPEN_SLEEP 5							  /* seconds to wait */
#define SB_OPEN_TRIES 60						  /* no of goes to have */

typedef struct word_struct

{
	String name;								  /* ascii representation */
	boolean found;								  /* seen it before? */
	String ptr;									  /* pointer to sample */
	int32 length;								  /* length of sample */
} word_struct, *word_struct_ptr;

/*
 * NOTE: KEEP THESE IN THE SAME ORDER AS IN LOADER.C !!!!
 */

typedef enum

{
	W_1, W_2, W_3, W_4, W_5, W_6, W_7, W_8, W_9, W_10, W_11, W_12,
	W_30, W_ITS, W_OCLOCK, W_AM, W_PM
} word_enum;

/* prototypes */

/* loader.c */

extern String read_voicebank(String);
extern boolean validate_voicebank(String, int *);
extern boolean extract_words(String);
extern boolean is_voc_file(String, int *);
extern String read_sample_file(String, int *);
extern int setup_voc_block(unsigned char *, int *, int *);
extern String get_sample_data(word_enum, int *);

#endif /* VOXCLOCK_H_INC */
